@@ -173,8 +173,9 @@ add_char (json_create_t * jc, unsigned char c)
 /* Add a nul-terminated string to "jc", up to the nul byte. This
    should not be used unless it's strictly necessary, prefer to use
    "add_str_len" instead. This is not intended to be Unicode-safe, it
-   is only to be used for strings which we know are not Unicode (for
-   example sprintf'd numbers or something). */
+   is only to be used for strings which we know do not need to be
+   checked for Unicode validity (for example sprintf'd numbers or
+   something). Basically, don't use this. */
 
 static INLINE json_create_status_t
 add_str (json_create_t * jc, const char * s)
@@ -191,7 +192,7 @@ add_str (json_create_t * jc, const char * s)
 /* Add a string "s" with length "slen" to "jc". This does not test for
    nul bytes, but just copies "slen" bytes of the string.  This is not
    intended to be Unicode-safe, it is only to be used for strings we
-   know are not Unicode. */
+   know do not need to be checked for Unicode validity. */
 
 static INLINE json_create_status_t
 add_str_len (json_create_t * jc, const char * s, unsigned int slen)
@@ -594,7 +595,7 @@ json_create_add_stringified (json_create_t * jc, SV *r)
     }
 
 /* Given a reference to a hash in "input_hv", recursively process it
-   into JSON. */
+   into JSON. "object" here means "JSON object", not "Perl object". */
 
 static INLINE json_create_status_t
 json_create_add_object (json_create_t * jc, HV * input_hv)
@@ -718,25 +719,18 @@ json_create_validate_user_json (json_create_t * jc, SV * json)
     PUSHMARK(SP);
     XPUSHs (sv_2mortal (newSVsv (json)));
     PUTBACK;
-/*
-    fprintf (stderr, "%s:%d: validating %s.\n", __FILE__, __LINE__,
-	     SvPV_nolen (json));
-*/
     call_pv ("JSON::Parse::assert_valid_json",
 	     G_EVAL|G_DISCARD);
     FREETMPS;
     LEAVE;  
-//    fprintf (stderr, "%s:%d: validating.\n", __FILE__, __LINE__);
     error = get_sv ("@", 0);
     if (! error) {
 	return json_create_ok;
     }
-//    fprintf (stderr, "%s:%d: validating %p.\n", __FILE__, __LINE__, error);
     if (SvOK (error) && SvCUR (error) > 0) {
-//	fprintf (stderr, "%s:%d: validating.\n", __FILE__, __LINE__);
-	croak ("JSON::Parse::assert_valid_json failed for '%s': %s", SvPV_nolen (json), SvPV_nolen (error));
+	croak ("JSON::Parse::assert_valid_json failed for '%s': %s",
+	       SvPV_nolen (json), SvPV_nolen (error));
     }
-//    fprintf (stderr, "%s:%d: validating.\n", __FILE__, __LINE__);
     return json_create_ok;
 }
 
@@ -744,25 +738,24 @@ json_create_validate_user_json (json_create_t * jc, SV * json)
 static json_create_status_t
 json_create_call_to_json (json_create_t * jc, SV * cv, SV * r)
 {
-//    fprintf (stderr, "Now that's a real mother for you, yeah.\n");
     I32 count;
     SV * json;
     char * jsonc;
     STRLEN jsonl;
-// https://metacpan.org/source/AMBS/Math-GSL-0.35/swig/gsl_typemaps.i#L438
+    // https://metacpan.org/source/AMBS/Math-GSL-0.35/swig/gsl_typemaps.i#L438
     dSP;
     
     ENTER;
     SAVETMPS;
     
     PUSHMARK(SP);
-//https://metacpan.org/source/AMBS/Math-GSL-0.35/swig/gsl_typemaps.i#L482
+    //https://metacpan.org/source/AMBS/Math-GSL-0.35/swig/gsl_typemaps.i#L482
     XPUSHs(sv_2mortal(newRV(r)));
     PUTBACK;
     count = call_sv (cv, 0);
     if (count != 1) {
-	fprintf (stderr, "not one but %ld .\n", count);
-	return json_create_ok;
+	warn ("Wrong number of arguments %ld from user callback: should be 1",
+	      count);
     }
     json = POPs;
     SvREFCNT_inc (json);
@@ -770,7 +763,6 @@ json_create_call_to_json (json_create_t * jc, SV * cv, SV * r)
     LEAVE;  
     jsonc = SvPV (json, jsonl);
     if (jc->validate) {
-//	fprintf (stderr, "validating.\n");
 	CALL (json_create_validate_user_json (jc, json));
     }
     CALL (add_str_len (jc, jsonc, jsonl));
@@ -799,10 +791,17 @@ json_create_handle_ref (json_create_t * jc, SV * input)
 	break;
 
     case SVt_PVMG:
-#ifdef DEBUGOBJ
-	fprintf (stderr, "monkey monkey.\n");
-#endif 
-	CALL (json_create_add_string (jc, input));
+	/* There are some edge cases with blessed references
+	   containing numbers which we need to handle correctly. */
+	if (SvIOK (r)) {
+	    CALL (json_create_add_integer (jc, r));
+	}
+	else if (SvNOK (r)) {
+	    CALL (json_create_add_float (jc, r));
+	}
+	else {
+	    CALL (json_create_add_string (jc, input));
+	}
 	break;
 
     case SVt_PVGV:
@@ -863,26 +862,21 @@ json_create_handle_object (json_create_t * jc, SV * input)
 		SV * what;
 		what = SvRV (*sv_ptr);
 		if (SvROK (what)) {
-//		    fprintf (stderr, "It's a ref baby.\n");
 		    what = SvRV(what);
 		}
-/*
-		fprintf (stderr, "Now what a real mother for you yeah %d.\n",
-			 SvTYPE (what));
-*/
 		switch (SvTYPE (what)) {
 		case SVt_PVCV:
 		    CALL (json_create_call_to_json (jc, what, r));
 		    break;
 		default:
-//		    fprintf (stderr, "Not code, looks like %s\n", SvPV_nolen (what));
+		    /* Weird handler, not a code reference. */
 		    goto nothandled;
 		}
 	    }
 	    else {
 		/* It's an object, it's in our handlers, but we don't
-	       have any code to deal with it, so we'll print an error
-	       and then stringify it. */
+		   have any code to deal with it, so we'll print an
+		   error and then stringify it. */
 		if (JCEH) {
 		    (*JCEH) (__FILE__, __LINE__, "Unhandled handler %s.\n",
 			     pv);
@@ -930,6 +924,8 @@ json_create_recursively (json_create_t * jc, SV * input)
 	ADD ("null");
 	return json_create_ok;
     }
+    /* JSON::Parse inserts pointers to &PL_sv_yes and no as literal
+       "true" and "false" markers. */
     if (input == &PL_sv_yes) {
 	ADD ("true");
 	return json_create_ok;
